function Multirun
%------------------------------- Copyright ------------------------------------
% Copyright (c) MDO Group, UNSW, Australia. You are free to use the SASSEA for
% research purposes. All publications which use this code should acknowledge the 
% use of "SASSEA" and reference "K.H.Rahi, H.K.Singh, T.Ray, A steady-state algorithm for 
% solving expensive multi-objective optimization problems with non-parallelizable evaluations, 
% IEEE Transactions on Evolutionary Computation, 2022 (Accepted for publication)".
%------------------------------------------------------------------------------ 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Multirun: This function is used to run specified algorithms and problems across multiple trials. 
%  Set the parameters first at the Params.m file properly.
%  Check that matlab path is added properly for all functions required
%  Now run this script.
%% Coded by 
%  Kamrul Hasan Rahi
%  k.rahi@student.adfa.edu.au; kamrulhasanme038@gmail.com
%  Last updated: October 26, 2022
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parallel Computing
clc;clear all;close all;warning off;
filename = mfilename;
motherfolderpath = which(filename);
motherfolder = fileparts(motherfolderpath);
cd(motherfolder);
addpath(genpath(motherfolder));
Params;path = cd;count = 1;

for i = 1:numel(param.allprobs)
    param.prob_name = param.allprobs{i}; 
    prob = feval(param.prob_name);
    if prob.nf < 3
        param.spacing = 99;
    elseif prob.nf == 3
        param.spacing = 12;
    elseif prob.nf == 5
        param.spacing = 4;
    elseif prob.nf == 7
        param.spacing = 3;
    end
    for j = 1:length(param.strategies)
        param.strategy = param.strategies(j);
        for k = 1:param.allruns(i)
            disp(strcat('Variant:- ',num2str(param.strategy),', Problem Name:- ',param.prob_name,', Trial:- ',num2str(k)));
            pth = strcat(cd,filesep,'Data',filesep,'Variant-',num2str(param.strategy),filesep,param.prob_name,filesep,'Trial-',num2str(k));
            mkdir(pth);
            cd(pth);
            param.seed = 100+k;
            param.MFE = param.MFE_all(i);
            save('Parameters.mat', 'param');
            path1{count} = pth;
            count = count+1;
            cd(path);
        end
    end
end
cd(path);

clear param;
%% parfor: utilize multiple core of PC.
% If you want to turn off parallel computing, then comment lines 62, 63 and
% 71. Then change 'parfor' to 'for' in line 64.
numcores = feature('numcores');
parpool(numcores);
parfor i = 1:length(path1)
    cd(path1{i});
    disp(strcat('Running -> ',path1{i}));
    tic;
    Driver(path1{i});
    toc;
end
delete(gcp);
cd(path);
return